"""
Copyright (c) Jordan Maxwell, All Rights Reserved.
See LICENSE file in the project root for full license information.
"""

from droiddepot.utils import *

class OfficialDroidBeaconLocations(object):
    """
    Constants representing every official Walt Disney World and DisneyLand SWGE Droid Beacon
    """
 
    DL_Marketplace =           '0A040102A601'
    DL_BehindDroidDepot =      '0A040202A601'
    DL_Resistence =            '0A040302A601'
    DL_FirstOrder =            '0A040702A601'
    DL_DroidDepot =            '0A040318BA01'
    DL_InFrontOfOgas =         '0A0405FFA601'
    DL_MarketplaceEntertance = '0A040502A601'

    WDW_OutdoorsArea =         '0A040102A601'
    WDW_BehindDroidDepot =     '0A040202A601'
    WDW_Resistence =           '0A040302A601'
    WDW_DokOndars =            '0A040602A601'
    WDW_FirstOrder =           '0A040702A601'
    WDW_Marketplace =          '0A040618BA01'
    WDW_DroidDetector =        '0A0405FFA601'
    WDW_InFrontOfOgas =        '0A0407FFA601'
